package top.ninwoo.dbc.cloud.client.service;

import top.ninwoo.bishe.starter.service.NetworkService;
import top.ninwoo.dbc.api.po.*;
import top.ninwoo.dbc.api.service.DistributedComService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import javax.annotation.PostConstruct;
import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.util.*;

@Service
public class FileCloService {

    @Value("${dbc.directory.output}")
    private String directoryOutput;

    @Qualifier(value="cloudRestTemplates")
    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private NetworkService networkService;

    @Autowired
    private DistributedComService distributedComService;

    //String cloudIp = "127.0.0.1:8888";
    //String cloudIp = "172.17.0.4:8888";

    private String cloudIp = null;

    @PostConstruct
    public void init() {
        cloudIp = getIpList(11114l,"dbc_cloud_server");
        System.out.println(cloudIp);
    }
    public String getIpList(Long clusterId, String appName) {
        List<String> ipList = networkService.getIpListByAppName(clusterId, appName);
        if(!ipList.isEmpty()){
            String ip_tmp = ipList.get(0);
            String[] split_list = ip_tmp.split("/");
            String containerIp = split_list[0]+":8888";
            return containerIp;
        }
        return null;
    }


    /**
     * 发送文件到云端
     * @param fileId
     * @param fileBytes
     * @return
     */
    @Async("taskExecutor")
    public String sendFile(Long fileId, byte[] fileBytes) {
        // 通过集群服务器接口获取当前集群的节点数量

        cloudIp = getIpList(11114l,"dbc_cloud_server");

        // todo 这里需要制定文件的fileId
        FileCloudSlice fileCloudSlice = new FileCloudSlice();
        fileCloudSlice.setFileCloudId(fileId);
        fileCloudSlice.setFileByte(fileBytes);

        String result = sendFileSlice(cloudIp, fileCloudSlice);
        if(result.equals("failed")) {
            return "fail";
        }
        return "success";

    }

    // 发送文件切片到目标地址
    public String sendFileSlice(String targetIp, FileCloudSlice fileCloudSlice) {
        ResponseEntity<String> response = restTemplate.postForEntity("http://" + targetIp + "/data/put/", fileCloudSlice, String.class);
        if (!response.getStatusCode().is2xxSuccessful()) {
            return "failed!";
        }
        return response.getBody();
    }


    public Set<FileCloudSlice> getFileSlice(String targetIp, Long fileId) {
        FileCloudSlice[] fileCloudSlices = restTemplate.getForObject("http://" + targetIp + "/data/get/" + fileId, FileCloudSlice[].class);
        HashSet<FileCloudSlice> fileCloudSliceSet = new HashSet<>();
        fileCloudSliceSet.addAll(Arrays.asList(fileCloudSlices));
        return fileCloudSliceSet;
    }

    public String getFile(String fileName, Long fileId) {
        cloudIp = getIpList(11114l,"dbc_cloud_server");

        Set<FileCloudSlice> result = new HashSet<>();

        Set<FileCloudSlice> fileCloudComSet = getFileSlice(cloudIp, fileId);
        result.addAll(fileCloudComSet);
        List<FileCloudSlice> list = new ArrayList<>(result);
        SendResult sendResult = new SendResult();
        sendResult.setFileCloudSliceList(list);

        CloudComputingResult cloudComputingResult = distributedComService.fileComputing(sendResult);

        try {
            saveFile(fileName, cloudComputingResult);
            return "success";
        } catch (Exception e) {
            e.printStackTrace();
            return "fail";
        }
    }


    public void saveFile(String fileName, CloudComputingResult cloudComputingResult) {

        byte[] bytes = cloudComputingResult.getFileComBytes();
        ByteArrayInputStream bis = new  ByteArrayInputStream(bytes);
        try {
            BufferedImage images = ImageIO.read(bis);
            ImageIO.write(images,  "jpg", new File(directoryOutput + fileName));
        } catch (IOException e) {
            e.printStackTrace();
        }
        try {
            bis.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
     }
}
